# -*- coding: utf-8  -*-
#
# Copyright (C) 2011-2025 Ben Kurtovic <ben.kurtovic@gmail.com>
# Released under the terms of the MIT License. See LICENSE for details.

"""
gitup: the git repository updater
"""

__author__ = "Ben Kurtovic"
__copyright__ = "Copyright (C) 2011-2025 Ben Kurtovic"
__license__ = "MIT License"
__version__ = "0.5.2"
__email__ = "ben.kurtovic@gmail.com"
